import logging
from ase.build import molecule
from ase.md.velocitydistribution import MaxwellBoltzmannDistribution
from ase.md.langevin import Langevin
import ase.io
import numpy as np

from pet_mad.calculator import PETMADCalculator

# Configure logging
logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
)

logging.info("Starting molecular dynamics simulation")

calc = PETMADCalculator()

n_steps = 10000

atoms = molecule("CO2")
atoms.calc = calc
MaxwellBoltzmannDistribution(atoms, temperature_K=300)

logging.info("Initialized CO2 molecule and set up calculator")

dyn = Langevin(
    atoms,
    timestep=0.5 * ase.units.fs,
    temperature_K=300.0,  # temperature in K
    friction=0.01 / ase.units.fs,
)

logging.info("Langevin dynamics initialized")


def print_step():
    logging.info(f"Step: {dyn.nsteps}")


dyn.attach(print_step, interval=10)

traj = ase.io.Trajectory("../data/co2_pet_mad_trajectory.traj", "w", atoms)
dyn.attach(traj.write)
logging.info("Trajectory file initialized")

dyn.run(n_steps)
logging.info("Molecular dynamics simulation completed")

traj = ase.io.read("../data/co2_pet_mad_trajectory.traj", ":")
dataset = [traj[i] for i in range(90000, len(traj), 10)]
ase.io.write("../data/co2_dataset.xyz", dataset, format="xyz")
logging.info("Dataset written to ../data/co2_dataset.xyz")


def generate_co2_traj(nsteps, bond_length, max_d=0.1):

    initial_mol = ase.Atoms(
        numbers=[6, 8, 8],
        positions=[
            [0.0, 0.0, 0.0],  # Carbon at origin
            [0.0, 0.0, bond_length],  # Oxygen 1
            [0.0, 0.0, -bond_length],  # Oxygen 2
        ],
    )

    traj = []
    for dz in np.linspace(-max_d, max_d, nsteps, endpoint=True):
        # Move the second oxygen atom
        new_mol = initial_mol.copy()
        new_mol.positions[1] = np.array([0.0, 0.0, bond_length + dz])
        traj.append(new_mol)
    return traj


logging.info("Generating CO2 trajectory with varying bond lengths")
bond_lengths = [mol.get_distance(0, 1) for mol in traj]
change_bond_length_traj = generate_co2_traj(
    100, bond_length=np.mean(bond_lengths), max_d=3 * np.std(bond_lengths)
)
ase.io.write(
    "../data/co2_change_bond_length.xyz", change_bond_length_traj, format="xyz"
)
logging.info(
    "Dataset with varying bond lengths written to ../data/co2_change_bond_length.xyz"
)
logging.info("All tasks completed successfully")
