import os
import logging
from pathlib import Path

import ase.io
import numpy as np
import pyscf.pbc.tools.pyscf_ase as pyscf_ase
from tqdm import tqdm
import jax.numpy as jnp
import jax

os.environ["PYSCFAD_BACKEND"] = "jax"

from pyscfad import scf, gto, ops  # noqa: F401, E402

# Configure logging
logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s - %(levelname)s - %(message)s",
)
logger = logging.getLogger(__name__)

basis = "aug-cc-pvdz"


def apply_E(mol, E):
    field = jnp.einsum("x,xij->ij", E, mol.intor("int1e_r"))
    mf = scf.RHF(mol)
    h1 = mf.get_hcore() + field
    mf.get_hcore = lambda *args, **kwargs: h1
    mf.kernel()
    return mf.e_tot


logger.info("Reading molecules from dataset...")
molecules = ase.io.read("../data/co2_dataset.xyz", ":")
out_folder = "outputs_TMP"
Path(out_folder).mkdir(parents=True, exist_ok=True)
E0 = jnp.zeros((3))  # a zero static electric field

logger.info("Processing molecules...")
for i in tqdm(range(0, len(molecules))):
    mol = gto.Mole(
        atom=pyscf_ase.ase_atoms_to_pyscf(molecules[i]), basis=basis, verbose=0
    )
    mol.build(trace_exp=False, trace_ctr_coeff=False)

    bec = jax.jacfwd(jax.grad(apply_E, 1), 0)(mol, E0).coords
    chi = -jax.jacfwd(jax.jacfwd(jax.grad(apply_E, 1), 1), 0)(mol, E0).coords

    Path(f"{out_folder}/{i}").mkdir(parents=True, exist_ok=True)
    ase.io.write(f"{out_folder}/{i}/molecule.xyz", molecules[i])
    np.save(f"{out_folder}/{i}/bec.npy", ops.to_numpy(bec))
    np.save(f"{out_folder}/{i}/raman.npy", ops.to_numpy(chi))

logger.info("Reading molecules with changed bond lengths...")
molecules_change_bond_length = ase.io.read("../data/co2_change_bond_length.xyz", ":")
out_folder = "outputs_change_bond_length_TMP"
Path(out_folder).mkdir(parents=True, exist_ok=True)
E0 = jnp.zeros((3))  # a zero static electric field

logger.info("Processing molecules with changed bond lengths...")
for i in tqdm(range(0, len(molecules_change_bond_length))):
    mol = gto.Mole(
        atom=pyscf_ase.ase_atoms_to_pyscf(molecules_change_bond_length[i]),
        basis=basis,
        verbose=0,
    )
    mol.build(trace_exp=False, trace_ctr_coeff=False)

    bec = jax.jacfwd(jax.grad(apply_E, 1), 0)(mol, E0).coords
    chi = -jax.jacfwd(jax.jacfwd(jax.grad(apply_E, 1), 1), 0)(mol, E0).coords

    Path(f"{out_folder}/{i}").mkdir(parents=True, exist_ok=True)
    ase.io.write(f"{out_folder}/{i}/molecule.xyz", molecules_change_bond_length[i])
    np.save(f"{out_folder}/{i}/bec.npy", ops.to_numpy(bec))
    np.save(f"{out_folder}/{i}/raman.npy", ops.to_numpy(chi))

logger.info("Processing complete.")
