from itertools import product
import numpy as np
import ase.io
from tqdm import tqdm
import logging

import metatensor as mts
from featomic.clebsch_gordan import cartesian_to_spherical

# Configure logging
logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
)

logging.info("Starting script execution.")


def parse_folders(folder_name, molecules, label):

    becs = []
    ramans = []

    logging.info("Start loading data")

    for idx in tqdm(range(len(molecules))):

        bec = np.load(f"{folder_name}/{idx}/bec.npy")
        raman = np.load(f"{folder_name}/{idx}/raman.npy")

        becs.append(bec)
        ramans.append(raman)

    logging.info("Finished loading data.")

    bec = -np.array(becs).transpose(0, 2, 1, 3)
    raman = np.array(ramans).transpose(0, 3, 1, 2, 4)

    bec_cartesian = mts.TensorMap(
        mts.Labels.single(),
        [
            mts.TensorBlock(
                samples=mts.Labels(
                    ["system", "atom"],
                    np.array(list(product(range(len(molecules)), range(3)))),
                ),
                components=[mts.Labels.range(f"xyz_{i}", 3) for i in range(2)],
                properties=mts.Labels.single(),
                values=bec.reshape(bec.shape[0] * bec.shape[1], *bec.shape[2:], 1),
            )
        ],
    )

    raman_cartesian = mts.TensorMap(
        mts.Labels.single(),
        [
            mts.TensorBlock(
                samples=mts.Labels(
                    ["system", "atom"],
                    np.array(list(product(range(len(molecules)), range(3)))),
                ),
                components=[mts.Labels.range(f"xyz_{i}", 3) for i in range(3)],
                properties=mts.Labels.single(),
                values=raman.reshape(
                    raman.shape[0] * raman.shape[1], *raman.shape[2:], 1
                ),
            )
        ],
    )

    logging.info("Converted data to cartesian TensorMaps.")

    bec_spherical = cartesian_to_spherical(bec_cartesian, bec_cartesian.component_names)
    bec_spherical = mts.drop_blocks(bec_spherical, mts.Labels.range("o3_lambda", 2))
    bec_spherical = mts.remove_dimension(bec_spherical, "keys", "_")

    raman_spherical = cartesian_to_spherical(
        raman_cartesian, [f"xyz_{i}" for i in range(3)]
    )
    raman_spherical = mts.drop_blocks(raman_spherical, mts.Labels.range("o3_lambda", 3))
    keys_to_remove = raman_spherical.keys.names[2:]
    for k in keys_to_remove:
        raman_spherical = mts.remove_dimension(raman_spherical, "keys", k)

    logging.info("Converted data to spherical TensorMaps.")

    mts.save(f"../data/bec{label}.mts", mts.make_contiguous(bec_spherical))
    logging.info(f"Saved bec_spherical to ../data/bec{label}.mts.")

    mts.save(f"../data/raman{label}.mts", mts.make_contiguous(raman_spherical))
    logging.info(f"Saved alpha_spherical to ../data/raman{label}.mts.")


folder_name = "outputs"
molecules = ase.io.read("../data/co2_dataset.xyz", index=":")
logging.info(f"Processing {folder_name}.")
parse_folders(folder_name, molecules, "_co2")

folder_name = "outputs_change_bond_length"
molecules_change_bond_length = ase.io.read(
    "../data/co2_change_bond_length.xyz", index=":"
)
logging.info(f"Processing {folder_name}.")
parse_folders(folder_name, molecules_change_bond_length, "_co2_change_bond_length")

logging.info("Script execution completed.")
