import ase.io
import torch
import logging

import metatensor.torch as mts
import metatensor.torch.atomistic as metatomic
from metatensor.torch.atomistic.ase_calculator import MetatensorCalculator

import sphericart.torch  # noqa: F401

# Set up logging
logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
)
logger = logging.getLogger(__name__)

device = "cuda" if torch.cuda.is_available() else "cpu"
logger.info(f"Using device: {device}")

outputs = {
    "mtt::bec": metatomic.ModelOutput(unit="a.u.", per_atom=True),
    "mtt::raman": metatomic.ModelOutput(unit="a.u.", per_atom=True),
}

logger.info("Loading MCoV model...")
MCoV_model = metatomic.load_atomistic_model(
    "../0_train/MCoV/model.pt",
    extensions_directory="../0_train/MCoV/extensions",
)
MCoV_model = MetatensorCalculator(MCoV_model, device=device)

logger.info("Loading lMCoV model...")
lMCoV_model = metatomic.load_atomistic_model(
    "../0_train/lMCoV/model.pt",
    extensions_directory="../0_train/lMCoV/extensions",
)
lMCoV_model = MetatensorCalculator(lMCoV_model, device=device)

logger.info("Reading CO2 bond length data...")
co2_change_bond_length = ase.io.read("../../data/co2_change_bond_length.xyz", ":")

logger.info("Running MCoV model predictions...")
MCoV_predictions = MCoV_model.run_model(co2_change_bond_length, outputs=outputs)

logger.info("Running lMCoV model predictions...")
lMCoV_predictions = lMCoV_model.run_model(co2_change_bond_length, outputs=outputs)

logger.info("Saving MCoV predictions...")
mts.save(
    "../../data/MCoV_predicted_bec_co2_change_bond_length.mts",
    MCoV_predictions["mtt::bec"].to(device="cpu", dtype=torch.float64),
)
mts.save(
    "../../data/MCoV_predicted_raman_co2_change_bond_length.mts",
    MCoV_predictions["mtt::raman"].to(device="cpu", dtype=torch.float64),
)

logger.info("Saving lMCoV predictions...")
mts.save(
    "../../data/lMCoV_predicted_bec_co2_change_bond_length.mts",
    lMCoV_predictions["mtt::bec"].to(device="cpu", dtype=torch.float64),
)
mts.save(
    "../../data/lMCoV_predicted_raman_co2_change_bond_length.mts",
    lMCoV_predictions["mtt::raman"].to(device="cpu", dtype=torch.float64),
)

logger.info("All tasks completed successfully.")
