import ase.io
import numpy as np
import matplotlib.pyplot as plt
import mpltex
import metatensor as mts

# Load atomic structure and data files
co2_change_bond_length = ase.io.read("data/co2_change_bond_length.xyz", ":")
MCoV_prediction_bec = mts.load("data/MCoV_predicted_bec_co2_change_bond_length.mts")
lMCoV_prediction_bec = mts.load("data/lMCoV_predicted_bec_co2_change_bond_length.mts")
MCoV_prediction_raman = mts.load("data/MCoV_predicted_raman_co2_change_bond_length.mts")
lMCoV_prediction_raman = mts.load(
    "data/lMCoV_predicted_raman_co2_change_bond_length.mts"
)
bec_reference = mts.load("data/bec_co2_change_bond_length.mts")
raman_reference = mts.load("data/raman_co2_change_bond_length.mts")


# Helper function to plot a single dataset
def plot_single(ax, dist, block, marker=None, color=None, ms=None, ls="-", lw=1):
    me = 10  # Marker frequency
    bl = block[0].values.reshape(100, 3, -1)  # Reshape tensor data
    idx = (bl.shape[-1] - 1) // 2  # Select middle index for plotting

    # Configure marker appearance
    markeredgewidth = 0.5 if marker is not None else None
    if marker == "x" and markeredgewidth is not None:
        markeredgewidth = 1
    markeredgecolor = "k" if marker is not None else None

    # Plot data for each atom
    for atm in range(3):
        ax.plot(
            dist,
            bl[:, atm, idx],
            ls=ls,
            lw=lw,
            color=color,
            marker=marker,
            markevery=me,
            markersize=ms,
            markeredgewidth=markeredgewidth,
            markeredgecolor=markeredgecolor,
        )


# Main plotting function with ACS style decorator
@mpltex.acs_decorator
def plot():
    figsize = plt.rcParams["figure.figsize"]  # Default figure size
    colors = {"ref": "black", "q": "tab:cyan", "l": "tab:red"}  # Line colors
    markers = {"ref": "x", "q": "v", "l": "o"}  # Marker styles
    ms_sizes = {"ref": 5, "q": 3, "l": 3}  # Marker sizes
    lw = 1  # Line width

    # Create subplots for BEC and Raman data
    fig, axes = plt.subplots(
        1, 2, dpi=250, figsize=(figsize[0] * 2, figsize[1]), sharex=True
    )

    d = np.linspace(-3, 3, 100, endpoint=True)  # Bond length range

    # Data configuration for each subplot
    data = [
        (
            axes[0],
            bec_reference,
            MCoV_prediction_bec,
            lMCoV_prediction_bec,
            r"$(Z^\star)^{(\lambda=2, \mu=0)}_{I\alpha\beta}$ (a.u.)",
            (-1.5, 2.5),  # Y-axis limits
        ),
        (
            axes[1],
            raman_reference,
            MCoV_prediction_raman,
            lMCoV_prediction_raman,
            r"$\chi^{(\lambda=3, \mu=0)}_{I\alpha\beta\gamma}$ (a.u.)",
            None,  # No Y-axis limits
        ),
    ]

    # Plot data for each subplot
    for ax, ref, q_pred, l_pred, ylabel, ylim in data:
        ax.grid(True, which="both", lw=0.5, ls="-", color="tab:gray", alpha=0.5)  # Grid

        # Plot reference and predictions
        for key, block in zip(["ref", "q", "l"], [ref, q_pred, l_pred]):
            plot_single(
                ax,
                d,
                block,
                color=colors[key],
                marker=markers[key],
                ms=ms_sizes[key],
                ls="-",
                lw=lw,
            )

        ax.set_ylabel(ylabel)  # Set Y-axis label
        if ylim:
            ax.set_ylim(ylim)  # Set Y-axis limits if provided

    # Annotate atoms on the first subplot
    axes[0].annotate(
        "C",
        (0.2, 0.0),
        (0, 0.9),
        arrowprops=dict(arrowstyle="->", lw=1),
        ha="center",
        va="center",
    )
    axes[0].annotate(
        "C",
        (0.2, 1.9),
        (0, 0.9),
        arrowprops=dict(arrowstyle="->", lw=1),
        ha="center",
        va="center",
    )
    axes[0].text(0, -0.78, "O$_1$, O$_2$")  # Label oxygen atoms

    # Annotate atoms on the second subplot
    axes[1].text(0, 0.2, "C")
    axes[1].text(0.1, 1.2, "O$_1$")
    axes[1].text(0.1, -0.9, "O$_2$")

    # Add legend to the second subplot
    axes[1].legend(
        handles=[
            plt.Line2D(
                [0],
                [0],
                color=colors["ref"],
                ls="-",
                marker=markers["ref"],
                markeredgewidth=1,
                markersize=ms_sizes["ref"],
                markeredgecolor="k",
                label=r"Ref.",
            ),
            plt.Line2D(
                [0],
                [0],
                color=colors["l"],
                ls="-",
                marker=markers["l"],
                markeredgewidth=0.5,
                markersize=ms_sizes["l"],
                markeredgecolor="k",
                label=r"$\lambda$-MCoV",
            ),
            plt.Line2D(
                [0],
                [0],
                color=colors["q"],
                ls="-",
                marker=markers["q"],
                markeredgewidth=0.5,
                markersize=ms_sizes["q"],
                markeredgecolor="k",
                label=r"MCoV",
            ),
        ],
        ncols=1,
        loc="lower left",
        bbox_to_anchor=(1.0, 0.0),
    )

    # Configure X-axis for both subplots
    for ax in axes:
        ax.set_xticks(range(-3, 4))  # Set X-axis ticks
        ax.set_xlim(-2, 2)  # Set X-axis limits

        ax.set_xlabel("Bond length")  # Set X-axis label
        ax.set_xticklabels(
            [
                r"$\overline{d}-3\sigma$",
                "",
                r"$\overline{d}-\sigma$",
                r"$\overline{d}$",
                r"$\overline{d}+\sigma$",
                "",
                r"$\overline{d}+3\sigma$",
            ]
        )  # Custom tick labels

    fig.tight_layout()  # Adjust layout to avoid overlap

    # Save the figure as a PDF
    fig.savefig("bec_and_raman_co2.pdf", bbox_inches="tight", transparent=True, dpi=600)


plot()  # Call the main plotting function
plt.close()
